<?php

namespace Slack\Blockkit\Blocks\Elements;

use Slack\Blockkit\Element;

/**
 * This is an element of an input dialog
 */
final class PlaintTextInput extends Element
{
	protected const LIMITS = [
		'action_id' => 255,			// @todo Should be unique for each message
		'placeholder' => 150,
	];

	private const MAX_MIN_LENGTH = 3000;

	// @todo dispatch_action_config
	// @todo focus_on_load

	public function __construct(string $action_id)
	{
		parent::__construct();

		$this->type = 'plain_text_input';

		$this->action_id = $this->validate('action_id',$action_id);
	}

	public static function item(string $action_id): self
	{
		return new self($action_id);
	}

	/* OPTIONAL ITEMS */

	public function initial_value(string $text): self
	{
		$this->initial_value = $text;

		return $this;
	}

	public function min_length(int $int): self
	{
		if ($int > self::MAX_MIN_LENGTH)
			throw new Exception(sprintf('min_length must be less than %d',self::MAX_MIN_LENGTH));

		$this->min_length = $int;

		return $this;
	}

	public function max_length(int $int): self
	{
		if ($this->min_length && ($int < $this->min_length))
			throw new Exception('max_length must be greater than min_length');

		$this->max_length = $int;

		return $this;
	}

	public function multiline(bool $bool): self
	{
		$this->multiline = $bool;

		return $this;
	}

	public function placeholder(Text $text): self
	{
		if (strlen($text->text) > self::LIMITS['placeholder'])
			throw new Exception(sprintf('Text must be %d chars or less',self::LIMITS['placeholder']));

		$this->placeholder = $text;

		return $this;
	}
}