<?php

namespace Slack\Blockkit\Blocks;

use \Exception;

use Slack\Blockkit\{Blocks,Element};

final class Input extends Blocks
{
	protected const LIMITS = [
		'block_id' => 255,			// @todo Should be unique for each message
		'hint' => 2000,
		'label' => 2000,
	];

	private const VALID_ELEMENTS = [
		Elements\PlaintTextInput::class,
		Elements\MultiStaticSelect::class
	];

	// @todo dispatch_action

	/**
	 * @param Elements\Text|null $label
	 * @throws Exception
	 */
	public function __construct(Elements\Text $label=NULL)
	{
		parent::__construct();

		if ($label->type != 'plain_text')
			throw new Exception(sprintf('Text must be plain_text not %s',$label->type));

		// Defaults
		$this->type = 'input';

		if (strlen($label->text) > self::LIMITS['label'])
			throw new Exception(sprintf('Text must be %d chars or less',self::LIMITS['label']));

		$this->label = $label;
	}

	public static function item(Elements\Text $label=NULL): self
	{
		return new self($label);
	}

	public function jsonSerialize()
	{
		if (! $this->element)
			throw new Exception('Must define an element');

		return parent::jsonSerialize();
	}

	/* OPTIONAL ITEMS */

	public function block_id(string $string): self
	{
		$this->block_id = $this->validate('block_id',$string);

		return $this;
	}

	public function element(Element $object): self
	{
		if (! in_array(get_class($object),self::VALID_ELEMENTS))
			throw new Exception(sprintf('Invalid element [%s] added to input',get_class($object)));

		$this->element = $object;

		return $this;
	}

	public function hint(Elements\Text $text): self
	{
		if (strlen($text->text) > self::LIMITS['hint'])
			throw new Exception(sprintf('Text must be %d chars or less',self::LIMITS['hint']));

		$this->hint = $text;

		return $this;
	}

	public function optional(bool $bool): self
	{
		$this->optional = $bool;

		return $this;
	}
}