<?php

namespace Slack\Blockkit\Blocks;

use \Exception;
use Illuminate\Support\Collection;

use Slack\Blockkit\{Blocks,Element};
use Slack\BLockKit\Blocks\Elements\Text;

final class Section extends Blocks
{
	protected const LIMITS = [
		'block_id' => 255,			// @todo Should be unique for each message
		'text' => 3000,
	];

	private const MAX_FIELDS = 10;
	private const MAX_FIELDS_TEXT = 2000;

	/**
	 * @param Text|NULL $text 			not required if fields is provided
	 * @throws Exception
	 */
	public function __construct(Text $text=NULL)
	{
		parent::__construct();

		// Defaults
		$this->type = 'section';

		if ($text) {
			if (strlen($text->text) > self::LIMITS['text'])
				throw new Exception(sprintf('Text must be %d chars or less',self::LIMITS['text']));

			$this->text = $text;
		}
	}

	public static function item(Text $text=NULL): self
	{
		return new self($text);
	}

	public function jsonSerialize()
	{
		if (! $this->text && ! $this->fields && ! $this->accessory)
			throw new Exception('Must define text, accessory or fields');

		return parent::jsonSerialize();
	}

	/* OPTIONAL ITEMS */

	public function accessory(Element $object): self
	{
		$this->accessory = $object;

		return $this;
	}

	public function block_id(string $string): self
	{
		$this->block_id = $this->validate('block_id',$string);

		return $this;
	}

	public function fields(Collection $collection): self
	{
		if (count($collection) > self::MAX_FIELDS)
			throw new Exception(sprintf('Can only have maximum %d fields',self::MAX_FIELDS));

		if (($x=$collection->map(function($item) { return strlen($item->text); })->max()) > self::MAX_FIELDS_TEXT)
			throw new Exception(sprintf('The maximum size of the text in a field is %d (%d)',self::MAX_FIELDS_TEXT,$x));

		$this->fields = $collection;

		return $this;
	}
}