<?php

namespace Slack\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

use App\Models\Slack\Channel;
use App\Models\Slack\Enterprise as AppEnterprise;
use App\Models\Slack\Team as AppTeam;
use Slack\Traits\ScopeActive;

class User extends Model
{
	use ScopeActive;

	protected const LOGKEY = '-MU';

	protected $fillable = ['user_id'];

	/* RELATIONS */

	public function enterprise()
	{
		return $this->belongsTo(class_exists(AppEnterprise::class) ? AppEnterprise::class : Enterprise::class);
	}

	public function team()
	{
		return $this->belongsTo(class_exists(AppTeam::class) ? AppTeam::class : Team::class);
	}

	/* ATTRIBUTES */

	/**
	 * Is this user an admin
	 *
	 * @return bool
	 */
	public function getIsAdminAttribute(): bool
	{
		return (bool)$this->admin;
	}

	/**
	 * Return the user in slack response format
	 */
	public function getSlackUserAttribute(): string
	{
		return sprintf('<@%s>',$this->user_id);
	}

	/**
	 * Return the team that this user is in - normally required to get the team token
	 * For enterprise users, any team token will do.
	 *
	 * If the integration is not installed in any channels, team will be blank
	 *
	 * @return mixed
	 */
	public function getUserTeamAttribute(): ?Team
	{
		Log::debug(sprintf('%s:User [%s]',self::LOGKEY,$this->id),['team'=>$this->team_id,'enterprise'=>$this->enterprise_id,'eo'=>$this->enterprise]);

		return $this->team_id ? $this->team : (($x=$this->enterprise->teams) ? $x->first() : NULL);
	}

	/* METHODS */

	/**
	 * Return if the user is allowed to use this bot
	 * If the user is active status is NULL, but the channel or team allow guests, then the user is allowed.
	 * If the user is active status is FALSE, then the user is not allowed.
	 *
	 * @param Channel $co
	 * @return bool
	 */
	public function isAllowed(Channel $co): bool
	{
		return $this->active || (($this->active !== FALSE) && $co->allow_guest);
	}
}